"""
Panflute filter to save a copy of the image to
the mediadir in non-standard mode
Move "src" to "data-src" for lazy loading
So if the theme doesn't implement lazy loading, you have to set the "src"
using javascript
"""
from pathlib import Path
from urllib.parse import unquote
import panflute as pf
from link import copy_to_media

def action(elem, doc):
    """action"""
    # local image or local link for download
    if isinstance(elem, pf.Image):
        # parse the attributes from href
        if '//' not in elem.url and doc.get_metadata('standalone') != '1':
            # local images
            filepath = Path(unquote(elem.url))
            if str(filepath)[:1] != '/':
                filepath = Path(doc.get_metadata('indir')) / filepath
            destfile = Path(doc.get_metadata('mediadir')) / filepath.name
            destfile = copy_to_media(filepath, destfile)
            elem.url = str(destfile.relative_to(doc.get_metadata('outdir')))
        elem.attributes['data-src'] = elem.url
        elem.url = 'data:null'

def main(doc=None):
    """main function"""
    return pf.run_filter(action, doc=doc)

if __name__ == '__main__':
    main()



"""Report generating system using pandoc"""
import re
from pathlib import Path
from functools import partial
from base64 import b64encode, b64decode
from shutil import rmtree
from cmdy import pandoc, wkhtmltopdf
from .filters.utils import copy_to_media

HERE = Path(__file__).resolve().parent
DEFAULT_FILTERS = ['table', 'link', 'image']
DEFAULT_TEMPLATE = 'layui'


def _replace_all(regex, callback, string):
    matches = re.finditer(regex, string)
    ret = ''
    start = 0
    for match in matches:
        ret += string[start:match.start(0)]
        start = match.end(0)
        ret += callback(match.group(), *match.groups())
    return ret + string[start:]

def _replace_ref_index(matched, citations):
    """citations: index => ref"""
    index = matched[2:-2]
    if index in citations:
        return '[#REF: %s #]' % citations[index]
    return matched

def _replace_ref(matched, citations):
    """citations: ref => index"""
    cite = matched[7:-3] # [#REF: ... #]
    if cite not in citations:
        citations[cite] = len(citations) + 1
    return '[[{i}]](#PYPPL-REF-{i}){{.pyppl-report-refindex}}'.format(
        i=citations[cite]
    )


class ProcReport:  # pylint: disable=too-few-public-methods
    """Report for each process"""
    def __init__(self, rptfile):
        self.rptfile = Path(rptfile)
        self.source = None

    def prepare(self):
        """Get the citations"""
        lines = self.rptfile.read_text().splitlines()
        # get the references first
        # references should be placed at the bottom
        source = []
        citations = {}
        for line in lines:
            line = line.rstrip('\n')
            # [1]: reference 1
            # [2]: reference 2
            matched = re.match(r'\[\[(\d+)\]\]: (.+)', line)
            if not matched:
                source.append(line)
                continue
            citations[matched.group(1)] = b64encode(
                matched.group(2).strip().encode()
            ).decode()

        source = '\n'.join(source) if source else ''
        self.source = _replace_all(r'\[\[\d+\]\]',
                                   partial(_replace_ref_index,
                                           citations=citations),
                                   source)

class Report:
    """The Report class"""
    def __init__(self, rptfiles, outfile, title='Untitled document'):
        self.reports = [ProcReport(rptfile) for rptfile in rptfiles]
        # // convert to relative path
        try:
            self.outfile = Path(outfile).resolve().relative_to(Path.cwd())
        except ValueError:
            self.outfile = Path(outfile)
        self.mdfile = self.outfile.with_suffix('.tmp.md')
        self.title = title

    def cleanup(self):
        """Cleanup and generate markdown for each process"""
        citations = {}

        with self.mdfile.open('w') as fmd:
            for report in self.reports:
                report.prepare()
                # replace reference to citation indexes
                source = _replace_all(r'\[#REF: .+? #\]',
                                      partial(_replace_ref,
                                              citations=citations),
                                      report.source)

                fmd.write(source + '\n\n')

            if citations:
                fmd.write('# Reference\n')
                for cite, index in sorted(citations.items(),
                                          key=lambda item: int(item[1])):
                    fmd.write('[[{i}]: {cite}](){{.pyppl-report-reference '
                              'name=PYPPL-REF-{i}}}'.format(
                                  i=index, cite=b64decode(cite).decode()
                              ))

    def generate(self, standalone, template, filters, toc):
        """Generate the reports"""
        from pyppl import __version__ as pyppl_version
        from . import __version__ as report_version

        self.cleanup()
        template = template or DEFAULT_TEMPLATE
        if template and '/' not in template:
            template = HERE / 'templates' / template / 'template.html'

        ext = self.outfile.suffix.lower()
        if 'htm' not in ext and 'pdf' not in ext:
            raise ValueError('Only html and pdf format supported currently.')
        if not standalone and 'pdf' in ext:
            raise ValueError('pdf format has to be standalone.')

        metadata = [
            'pagetitle=%s' % self.title,
            'pyppl_version=%s' % pyppl_version,
            'report_version=%s' % report_version,
            'standalone=%s' % int(standalone),
            'indir=%s' % self.reports[0].rptfile.parent if self.reports else '',
            'outdir=%s' % self.outfile.parent,
            'pdf=%s' % bool('pdf' in ext)
        ]
        if toc > 0:
            metadata.append('toc=%d' % toc)
        mediadir = self.outfile.with_suffix('.files')
        srcpath = ['.', str(self.outfile.parent), str(mediadir),
                   str(Path(template).parent)]
        # add input directory
        srcpath += [str(self.reports[0].rptfile.parent)] if self.reports else []
        if mediadir.is_dir():
            rmtree(mediadir)
        metadata.extend(['mediadir=%s' % mediadir, 'template=%s' % template])

        if not standalone:
            # copy files to dir relative to the output file
            dest_staticdir = self.outfile.parent.joinpath('static')
            src_staticdir = Path(template).parent.joinpath('static')
            if not src_staticdir.is_dir():
                raise ValueError(
                    'Directory "static" not found in the template directory'
                )
            dest_staticdir.mkdir(parents=True, exist_ok=True)
            for staticfile in src_staticdir.rglob('*'):
                if staticfile.is_file():
                    copy_to_media(staticfile,
                                  dest_staticdir.joinpath(
                                      staticfile.relative_to(src_staticdir)
                                  ))

        kwargs = {
            'metadata': metadata,
            'read': 'markdown',
            'write': 'html5',
            'template': template,
            'filter': [HERE.joinpath('filters', filt + '.py')
                       if '_' not in filt
                       else filt
                       for filt in DEFAULT_FILTERS + (filters or [])],
            'self-contained': standalone,
            'resource-path': ':'.join(srcpath),
            '_raise': True,
            '_sep': 'auto',
            '_dupkey': True,
        }
        if 'pdf' not in ext:
            kwargs['output'] = self.outfile
            kwargs['_hold'] = True
            return pandoc(self.mdfile, **kwargs)

        kwargs['_pipe'] = True
        return pandoc(self.mdfile, **kwargs) | wkhtmltopdf(
            '-',
            _=self.outfile,
            _raise=True,
            _hold=True
        )



"""
Panflute filter to parse CSV in fenced YAML code blocks

Turn
```table
caption : An awesome table
file    : /path/to/csv/tsv/file
header  : true
download: true
rows    : 10
cols    : 0
sort    : [0]
pagesize: 25
csvargs :
    delimiter: "\t"
```

into:

```html
<div class="pyppl-report-table"
       data-prtable-caption="An awesome table"
       data-prtable-header="['HEADER1', 'HEADER2', ...]">
       data-prtable-data="[[ROW1], [ROW2], ...]"
       data-prtable-sort="[0]"
       data-prtable-download="true"
       data-prtable-file="FILE-PATH-TO-THE-TABLE"
       data-prtable-pagesize="25"></div>
```
which will be ready for the theme js to parse and render
"""
import io
import json
import csv
from pathlib import Path
import panflute as pf
from utils import copy_to_media, md5str

# pylint: disable=unused-argument,too-many-branches,too-many-statements
def fenced_action(options, data, element, doc):
    """fenced action"""
    # We'll only run this for CodeBlock elements of class 'table'

    attributes = {}
    attributes['data-prtable-file'] = ""
    attributes['data-prtable-download'] = str(options.get('download',
                                                          False)).lower()
    attributes['data-prtable-pagesize'] = str(options.get('sort', 25))
    attributes['data-prtable-sort'] = json.dumps(options.get('sort', [0]))
    attributes['data-prtable-caption'] = options.get('caption',
                                                     'Untitled Table')

    options['header'] = options.get('header', True)
    options['nrows'] = options.pop('rows', options.get('nrows', 0))
    options['ncols'] = options.pop('cols', options.get('ncols', 0))
    mediadir = Path(doc.get_metadata('mediadir'))
    options['nrows'] += 1 if options['nrows'] and options['header'] else 0

    csvargs = options.get('csvargs', {})
    csvargs['dialect'] = csvargs.get('dialect', "unix")
    csvargs['delimiter'] = csvargs.get('delimiter',
                                       "\t").encode().decode('unicode_escape')
    comment = csvargs.pop('comment', False)

    options['file'] = (Path(doc.get_metadata('indir')) / options['file']
                       if 'file' in options and str(options['file'])[:1] != '/'
                       else options.get('file'))
    fdata = io.StringIO(data) if data else open(options['file'])
    attributes['data-prtable-header'] = False

    if comment:
        while True:
            fcursor = fdata.tell()
            pf.debug(fcursor)
            if fdata.readline().startswith(comment):
                continue
            fdata.seek(fcursor)
            break
    try:
        reader = csv.reader(fdata, **csvargs)
        attributes['data-prtable-data'] = []
        for i, row in enumerate(reader):
            if options['header'] and not attributes['data-prtable-header']:
                attributes['data-prtable-header'] = row
                continue

            if options['nrows'] and i > options['nrows']:
                break
            attributes['data-prtable-data'].append([
                x for k, x in enumerate(row)
                if not options['ncols'] or k < options['ncols']
            ])
    finally:
        fdata.close()

    if (attributes['data-prtable-header']
            and attributes['data-prtable-data']
            and len(attributes['data-prtable-data'][0]) ==
            len(attributes['data-prtable-header']) + 1):
        attributes['data-prtable-header'].insert(0, "ROWNAME")
    if (attributes['data-prtable-header']
            and attributes['data-prtable-header'][0] == ""):
        attributes['data-prtable-header'][0] = "ROWNAME"

    if attributes['data-prtable-download'] == 'true':
        if not options['file']:
            options['file'] = mediadir.joinpath(
                str(md5str(data))
            ).with_suffix('.txt')
            attributes['data-prtable-file'] = str(options['file'].relative_to(
                doc.get_metadata('outdir')
            ))
            with options['file'].open('w') as fdata:
                if attributes['data-prtable-header']:
                    fdata.write("\t".join(attributes['data-prtable-header']) +
                                "\n")
                for row in attributes['data-prtable-data']:
                    fdata.write("\t".join(row) +
                                "\n")
        else:
            # copy file to mediadir and check if file exists
            options['file'] = Path(options['file'])
            attributes['data-prtable-file'] = str(copy_to_media(
                options['file'],
                mediadir / options['file'].name
            ).relative_to(doc.get_metadata('outdir')))

    attributes['data-prtable-header'] = json.dumps(
        attributes['data-prtable-header']
    )
    attributes['data-prtable-data'] = json.dumps(
        attributes['data-prtable-data']
    )
    return pf.Div(classes=['pyppl-report-table'], attributes=attributes)

def main(doc=None):
    """main function"""
    return pf.run_filter(pf.yaml_filter,
                         tag='table',
                         function=fenced_action,
                         doc=doc)


if __name__ == '__main__':
    main()



"""A report generating system for PyPPL"""
import sys
from time import time
from hashlib import sha256
from pathlib import Path
import textwrap
import toml
from pyppl.plugin import hookimpl
from pyppl.logger import Logger
from pyppl.jobmgr import STATES
from pyppl.utils import fs, format_secs, filesig
from pyppl.exception import ProcessAttributeError
from diot import Diot, OrderedDiot
from cmdy import CmdyReturnCodeException
from .report import Report

__version__ = "0.6.1"

logger = Logger(plugin='report') # pylint: disable=invalid-name

def report_template_converter(value):
    """Convert relative path of a template to absolute"""
    if value and value.startswith('file:'):
        scriptpath = Path(value[5:])
        if not scriptpath.is_absolute():
            from inspect import getframeinfo, stack

            # 0: .../pyppl_report/pyppl_report/__init__.py
            # 1: .../PyPPL/pyppl/plugin.py
            # 2: .../site-packages/diot.py
            # 3: .../pyppl_report/tests/test_report.py
            # if it fails in the future, check if the callstacks
            # changed from pluggy
            caller = getframeinfo(stack()[3][0])
            scriptdir = Path(caller.filename).parent.resolve()
            scriptpath = scriptdir / scriptpath
        if not scriptpath.is_file():
            raise ProcessAttributeError(
                'Report template file does not exist: %s' % scriptpath)
        return "file:%s" % scriptpath
    return value


@hookimpl
def setup(config):
    """Setup the plugin"""
    config.config.report_template = ''
    config.config.report_envs = Diot(level=1, pre='', post='')


@hookimpl
def logger_init(logger):  # pylint: disable=redefined-outer-name
    """Add log level"""
    logger.add_level('report')


@hookimpl
def proc_init(proc):
    """Add config"""
    proc.add_config('report_template',
                    default='',
                    converter=report_template_converter)
    proc.add_config('report_envs',
                    default=Diot(),
                    converter=lambda envs: envs or Diot())


@hookimpl
def proc_postrun(proc, status):
    """Generate report for the process"""
    # skip if process failed or cached
    # pylint: disable=too-many-locals,too-many-branches,too-many-statements
    report_file = proc.workdir.joinpath('proc.report.md')
    template = proc.config.report_template
    template_file = None
    if template and template.startswith('file:'):
        template_file = Path(template[5:])
        logger.debug("Using report template: %s", template_file, proc=proc.id)

    if not template and report_file.is_file():
        report_file.unlink()

    if not template or status == 'failed':
        return

    signature = OrderedDiot([(key, value)
                             for key, value in sorted(proc.config.items())
                             if key.startswith('report_')])
    if template_file and template_file.is_file():
        signature.template = filesig(template_file)
    signature = sha256(toml.dumps(signature).encode()).hexdigest()

    if status == 'cached' and report_file.is_file():
        with report_file.open() as frpt:
            if frpt.readline().strip() == '<!--- %s -->' % signature:
                logger.debug("Report markdown file cached, skip.", proc=proc.id)
                return

    fs.remove(report_file)
    logger.debug('Rendering report template ...', proc=proc.id)
    if template_file:
        template = template_file.read_text()

    template = proc.template(textwrap.dedent(template), **proc.envs)
    rptdata = dict(jobs=[None] * proc.size, proc=proc, args=proc.args)
    for i, job in enumerate(proc.jobs):
        rptdata['jobs'][i] = job.data.job.copy()
        rptdata['jobs'][i]['i'] = job.data.i
        rptdata['jobs'][i]['o'] = job.data.o

        datafile = job.dir / 'output/job.report.data.toml'
        if datafile.is_file():
            with datafile.open() as fdata:
                rptdata['jobs'][i].update(toml.load(fdata))

    rptenvs = Diot(level=1, pre='', post='', title=proc.desc)
    rptenvs.update(proc.config.report_envs)
    rptdata['report'] = rptenvs
    try:
        reportmd = template.render(rptdata)
    except Exception as exc:
        raise RuntimeError('Failed to render report markdown for process: %s' %
                           (proc)) from exc
    reportmd = reportmd.splitlines()

    codeblock = False
    for i, line in enumerate(reportmd):
        if line.startswith('#') and not codeblock:
            reportmd[i] = '#' * (rptenvs.level - 1) + line
        elif codeblock:
            if line.startswith('```') and len(line) - len(
                    line.lstrip('`')) == codeblock:
                codeblock = False
        elif line.startswith('```'):
            codeblock = len(line) - len(line.lstrip('`'))

    report_file.write_text(
        '<!--- %s -->' % signature +
        proc.template(textwrap.dedent(rptenvs.pre),
                      **proc.envs).render(rptdata) + '\n\n' +
        '\n'.join(reportmd) + '\n\n' +
        proc.template(textwrap.dedent(rptenvs.post),
                      **proc.envs).render(rptdata) + '\n'
    )

def report(ppl, # pylint: disable=too-many-arguments
           outfile=None,
           title=None,
           standalone=True,
           template=False,
           filters=False,
           toc=3):
    """@API
    Generate report for the pipeline.
    Currently only HTML format supported.
    @params:
        outfile (file): The report file, could be a directory,
            then the report file will be:
            - `<pipeline name>.report.html`
        title (str): The title of the report.
            - Default: 'A report generated by pipeline powered by PyPPL'
        standalone (bool): A standalone html file? Default: True
            - Otherwise, static files, including js/css/images will be
              store in a separated directory.
    """
    timer = time()
    default_basename = '%s.report.html' % ppl.name
    if not outfile:
        outfile = Path('.').joinpath(default_basename)
    elif Path(outfile).is_dir():
        outfile = Path(outfile).joinpath(default_basename)
    else:
        outfile = Path(outfile)

    logger.report('Generating report using pandoc ...')
    reports = [
        proc.workdir.joinpath('proc.report.md') for proc in ppl.procs
        if proc.workdir.joinpath('proc.report.md').exists()
    ]

    # see if it is cached:
    if (outfile.is_file()
            and reports
            and outfile.stat().st_mtime >= max(rptmd.stat().st_mtime
                                               for rptmd in reports)):
        logger.report('Report cached: %s', outfile)
    else:
        # force to add a title.
        title = title or 'Reports for ' + ppl.name + ' pipeline'
        cmd = Report(reports, outfile, title).generate(standalone,
                                                       template,
                                                       filters,
                                                       toc)
        try:
            logger.debug('Running: ' + cmd.cmd)
            cmd.run()
            logger.report('Report generated: %s', outfile)
            logger.report('Time elapsed: %s', format_secs(time() - timer))
        except CmdyReturnCodeException as ex:
            ex = str(ex)
            if 'Stack space overflow' not in ex:
                logger.error(str(ex))
                sys.exit(1)

            logger.warning('Error "Stack space overflow" from pandoc, '
                           'try to use "+RTS -K512M -RTS"')
            cmds = cmd.cmd.split(' ', 1)
            cmds.insert(1, '+RTS -K512M -RTS')
            cmd.cmd = ' '.join(cmds)
            try:
                logger.debug('Running: ' + cmd.cmd)
                cmd.done = False
                cmd.run()
                logger.report('Report generated: %s', outfile)
                logger.report('Time elapsed: %s', format_secs(time() - timer))
            except CmdyReturnCodeException as ex2:
                logger.error(str(ex2))
                sys.exit(1)

@hookimpl
def pyppl_init(ppl):
    """Add method to PyPPL instance"""
    ppl.add_method(report, require='run')


@hookimpl
def cli_addcmd(commands):
    """Add subcommand to pyppl"""
    commands.report = 'Convert a Markdown file to report.'
    params = commands.report
    params['in'].required = True
    params['in'].desc = 'The input file.'
    params.i = params['in']
    params.i.type = list
    params.out.desc = 'The output file. Default: <in>.html'
    params.out.callback = lambda opt, ps: (
        opt.set_value(Path(ps.i.value[0]).with_suffix('.html'))
        if not opt.value
        else None
    )
    params.o = params.out
    params.nonstand = False
    params.nonstand.desc = ('Non-standalone mode. '
                            'Save static files in `<filename of --out>.files` '
                            'separately.')
    params.n = params.nonstand
    params.filter = []
    params.filter.desc = 'The filters for pandoc'
    params.toc = 3
    params.toc.desc = 'The depth of heading levels to put in TOC. 0 to disable.'
    params.title = 'Untitled document'
    params.title.desc = ['The title of the document.',
                         'If the first element of the document is H1 (#), '
                         'this will be ignored and the text of H1 will be '
                         'used as title.',
                         'If the title is specified as "# Title", '
                         'then a title will be added anyway.']
    params.template = 'bootstrap'
    params.template.desc = ('The template to use. '
                            'Either standard template name or '
                            'full path to template file.')


@hookimpl
def cli_execcmd(command, opts):
    """Run the command"""
    if command == 'report':
        cmd = Report(opts.i, opts.o,
                     opts.title).generate(not opts.nonstand, opts.template,
                                          opts.filter, opts.toc)
        try:
            logger.info('Running: ' + cmd.pipedcmd)
            cmd.run()
            logger.info('Report generated: ' + str(opts.o))
        except CmdyReturnCodeException as ex:
            logger.error(str(ex))
            sys.exit(1)



"""Some utilities for pyppl_report"""
from hashlib import md5
from pathlib import Path
from shutil import copyfile

def md5sum(filepath):
    """Get the md5sum of a file"""
    if not filepath or not Path(filepath).exists():
        return None
    return md5(Path(filepath).read_bytes()).hexdigest()

def md5str(string):
    """Get the md5 of a string"""
    return md5(string.encode('utf-8')).hexdigest()

def copy_to_media(filepath, destfile):
    """Copy files to mediadir, check their md5sum"""

    dest_md5 = md5sum(destfile)
    if dest_md5 and dest_md5 == md5sum(filepath):
        return destfile

    destfile = Path(destfile)
    # just overwrite asset files
    if dest_md5 and destfile.suffix not in ('.js', '.css', '.woff2', '.woff',
                                            '.eot', '.ttf'):
        candfiles = list(destfile.parent.glob("[[]*[]]" + destfile.name))
        if not candfiles:
            destfile = destfile.parent / ('[1]' + destfile.name)
        else:
            maxnum = max(
                int(cfile.name.split(']')[0][1:]) for cfile in candfiles)
            destfile = destfile.parent / ('[{}]{}'.format(
                maxnum + 1, destfile.name))

    destfile.parent.mkdir(exist_ok=True, parents=True)
    if destfile.is_symlink(): # pragma: no cover
        destfile.unlink()
    copyfile(str(filepath), str(destfile))
    return destfile



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyppl_report', 'pyppl_report.filters']

package_data = \
{'': ['*'],
 'pyppl_report': ['templates/bootstrap/*',
                  'templates/bootstrap/static/*',
                  'templates/layui/*',
                  'templates/layui/static/*',
                  'templates/layui/static/css/*',
                  'templates/layui/static/css/modules/*',
                  'templates/layui/static/css/modules/laydate/default/*',
                  'templates/layui/static/css/modules/layer/default/*',
                  'templates/layui/static/font/*',
                  'templates/layui/static/images/face/*',
                  'templates/layui/static/lay/*',
                  'templates/layui/static/lay/modules/*',
                  'templates/layui/static/lay/modules/mobile/*',
                  'templates/semantic/*',
                  'templates/semantic/static/*',
                  'templates/semantic/static/themes/default/assets/fonts/*',
                  'templates/semantic/static/themes/default/assets/images/*']}

install_requires = \
['cmdy', 'liquidpy', 'panflute>=1.11.0,<1.12.0', 'pyppl', 'toml>=0.10,<0.11']

entry_points = \
{'pyppl': ['pyppl_report = pyppl_report']}

setup_kwargs = {
    'name': 'pyppl-report',
    'version': '0.6.1',
    'description': 'A report generating system for PyPPL',
    'long_description': '# pyppl_report\n\n[![Pypi][3]][4] [![Github][5]][6] [![PyPPL][7]][1] [![PythonVers][8]][4] [![Travis building][10]][11] [![Codacy][12]][13] [![Codacy coverage][14]][13]\n\nA report generating system for [PyPPL][1]\n\n## Installation\nRequires pandoc 2.7+ (and wkhtmltopdf 0.12.4+ when creating PDF reports)\n\n`pyppl_report` requires `pandoc/wkhtmltopdf` to be installed in `$PATH`\n\n```shell\npip install pyppl_report\n```\n\n## Usage\n### Specifiation of template\n\n````python\npPyClone.config.report_template = """\n# {{report.title}}\n\nPyClone[1] is a tool using Probabilistic model for inferring clonal population\nstructure from deep NGS sequencing.\n\n![Similarity matrix]({{path.join(job.o.outdir, "plots/loci/similarity_matrix.svg")}})\n\n```table\ncaption: Clusters\nfile: "{{path.join(job.o.outdir, "tables/cluster.tsv")}}"\nrows: 10\n```\n\n[1]: Roth, Andrew, et al. "PyClone: statistical inference of clonal population structure in cancer."\nNature methods 11.4 (2014): 396.\n"""\n\n# or use a template file\n\npPyClone.config.report_template = "file:/path/to/template.md"\n````\n\n### Generating report\n```python\nPyPPL().start(pPyClone).run().report(\n\t\'/path/to/report\',\n\ttitle=\'Clonality analysis using PyClone\',\n\ttemplate=\'bootstrap\'\n)\n\n# or save report in a directory\nPyPPL(name=\'Awesome-pipeline\').start(pPyClone).run().report(\'/path/to/\')\n# report generated at ./Awesome-pipeline.report.html\n```\n\nCommand line tool:\n```shell\n> pyppl report\nDescription:\n  Convert a Markdown file to report.\n\nUsage:\n  pyppl report --in <LIST> [OPTIONS]\n\nRequired options:\n  -i, --in <LIST>           - The input file.\n\nOptional options:\n  -o, --out <AUTO>          - The output file. Default: <in>.html\n  -n, --nonstand [BOOL]     - Non-standalone mode. Save static files in  <filename of --out>.files  separately. \\\n                              Default: False\n      --filter <LIST>       - The filters for pandoc Default: []\n      --toc <INT>           - The depth of heading levels to put in TOC. 0 to disable. Default: 3\n      --title <STR>         - The title of the document.\n                              If the first element of the document is H1 (#), this will be ignored \\\n                              and the text of H1 will be used as title.\n                              If the title is specified as "# Title", then a title will be added \\\n                              anyway. Default: Untitled document\n      --template <STR>      - The template to use. Either standard template name or full path to \\\n                              template file. Default: bootstrap\n  -h, -H, --help            - Show help message and exit.\n```\n\n\n### Extra data for rendering\nYou can generate a `toml` file named `job.report.data.toml` under `<job.outdir>` with extra data to render the report template. Beyond that, `proc` attributes and `args` can also be used.\n\nFor example:\n`job.report.data.toml`:\n```toml\ndescription = \'A awesome report for job 1\'\n```\nThen in your template, you can use it:\n```markdown\n## {{jobs[0].description}}\n```\n\n## Built-in templates\n\nCheck them to see features those templates support:\n\n- [Layui](https://pwwang.github.io/pyppl_report/layui.html)\n- [Bootstrip](https://pwwang.github.io/pyppl_report/bootstrap.html)\n- [Semantic](https://pwwang.github.io/pyppl_report/semantic.html)\n\n\n## How does it work?\n\nFollowing figure demonstrates how the plugin works:\n\n![How it works](./docs/howitworks.png)\n\nEach process that you want to report, will need to have a template assigned with `pXXX.config.report_template`. Like scripts, you may prefice it with `file:`, and then followed by an absolute path to the template or a relative one to where it\'s assigned. You may even assign a template using a direct string. A process with no template assign will be hidden from the report.\n\nYou can use the data from the jobs or the process to render the template.\n\nThe report for each process will then be assembled by the plugin, and converted using pandoc with a default template and some built-in filters. Finally, your report will be a standalone html file.\n\nFor larget reports, `non-standaone` reports are recommended: `.report(standalone=False, ...)`\n\n## Environments\n\nYou may pass values to process envs to control report content:\n```python\npXXX.config.report_envs.foo = "bar"\n```\nThen in you can use it in the report template:\n```python\npXXX.config.report_template = """\nThe value of foo is "{{foo}}".\n"""\n```\n\n### Preserved envs variables\n\nWe have 4 preserved variables under `pXXX.envs`:\n```python\n# Control the level of headings in the\npXXX.config.report_envs.level = 1\n# Content to add before the template\npXXX.config.report_envs.pre = \'\'\n# Content to add after the template\npXXX.config.report_envs.post = \'\'\n# The title of the process report\npXXX.config.report_envs.title = None\n```\n\n#### Process report levels\n\nNo matter at which level you want to put this process report in the entire report, you need to each heading from level 1, then according to `pXXX.config.report_envs.level`, the headings will be shifted to corresponding level. For example, with `pXXX.config.report_envs.level = 2`, following template\n\n```markdown\n# Section\n## Subsection\ncontent\n```\n\nwill be rendered into:\n```markdown\n## Section\n### Subsection\ncontent\n```\n\nIt will not affect comments in code blocks such as:\n````\n```\n## some comments\n```\n````\n\n#### Adding extra contents to process report\n\nYou may add extra contents to the process report. For example, if you put the process report at level 2, then you probably need a level-1 heading. For previous example, if you have `pXXX.config.report_envs.level = 2`, without a level-2 heading, the entire report will look like:\n\n```markdown\n## Section\n### Subsection\ncontent\n```\n\nThen you missed a level-1 heading, which will make your report look wired. Here you can specify a level-1 heading with `pXXX.config.report_envs.pre = \'# I am H1\'`:\n\n```markdown\n# I am H2\n## Section\n### Subsection\ncontent\n```\n\nYou may also append something to the process report with `pXXX.config.report_envs.post`\n\nHeadings added by `pre` and `post` will **NOT** be adjusted by `pXXX.config.report_envs.level`\n\n#### Title of the process report\n\nBy default, if not assigned or assigned with `None`, the process description will be used as the title of the process report. Of course you can overwrite it with `pXXX.config.report_envs.title`.\n\n```python\n# by default\npXXX = Proc(desc = \'Some analysis\')\n# ... other necessary settings\npXXX.report = \'# {{report.title}}\'\n```\n\nwill be rendered as:\n```markdown\n# Some analysis\n```\n\nwith `pXXX.config.report_envs.title = \'An amazing analysis\'`, we will have:\n\n```markdown\n# An amazing analysis\n```\n\n### Making your report flexiable\n\nYou can interpolate some variables in the templates to make your report flexiable. For example, you may want to hide an image in some cases:\n\n```markdown\n# {{report title}}\n\nI have enough details.\n\n{% if report.get(\'showimage\') %}\n![Image](./path/to/image)\n{% endif %}\n```\n\nThen you can show that image in the report only when you have `pXXX.config.report_envs.showimage = True`.\n\n## Change log\n\n[Change log](./docs/CHANGELOG.md)\n\n[1]: https://github.com/pwwang/PyPPL\n[2]: https://pyppl_report.readthedocs.io/en/latest/\n[3]: https://img.shields.io/pypi/v/pyppl_report?style=flat-square\n[4]: https://pypi.org/project/pyppl_report/\n[5]: https://img.shields.io/github/tag/pwwang/pyppl_report?style=flat-square\n[6]: https://github.com/pwwang/pyppl_report\n[7]: https://img.shields.io/github/tag/pwwang/pyppl?label=PyPPL&style=flat-square\n[8]: https://img.shields.io/pypi/pyversions/pyppl_report?style=flat-square\n[9]: https://img.shields.io/readthedocs/pyppl_report.svg?style=flat-square\n[10]: https://img.shields.io/travis/pwwang/pyppl_report?style=flat-square\n[11]: https://travis-ci.org/pwwang/pyppl_report\n[12]: https://img.shields.io/codacy/grade/2b7914a18f794248a62d7b36eb2408a3?style=flat-square\n[13]: https://app.codacy.com/manual/pwwang/pyppl_report/dashboard\n[14]: https://img.shields.io/codacy/coverage/2b7914a18f794248a62d7b36eb2408a3?style=flat-square\n',
    'author': 'pwwang',
    'author_email': 'pwwang@pwwang.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pwwang/pyppl_report',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



"""
Panflute filter to support attributes for in Markdown

Turn
```markdown
[TEXT](URL {attr1=a, attr2=b} "TITLE")
```

into:

```html
<a class="pyppl-report-link"
   data-prlink-attr1="a"
   data-prlink-attr2="b"
   title="TITLE">TEXT</a>
```
which will be ready for the theme js to parse and render

If we are in non-standalone mode, try to save a copy of the file to mediadir
for the downloadable link
"""
from pathlib import Path
from urllib.parse import unquote
import panflute as pf
from utils import copy_to_media

def action(elem, doc):
    """action"""
    # local image or local link for download
    if isinstance(elem, pf.Link):
        attrs = elem.attributes.__class__([
            ('prlink-' + attrkey,
             attrval.lower()
             if attrval.lower() in ('true', 'false')
             else attrval)
            for attrkey, attrval in elem.attributes.items()
        ])
        if attrs.get('prlink-download') == 'true':
            filepath = Path(unquote(elem.url))
            if str(filepath)[:1] != '/':
                filepath = Path(doc.get_metadata('indir')) / filepath
            dstfile = Path(doc.get_metadata('mediadir')) / filepath.name
            dstfile = copy_to_media(filepath, dstfile)
            elem.url = str(dstfile.relative_to(doc.get_metadata('outdir')))

def main(doc=None):
    """main function"""
    return pf.run_filter(action, doc=doc)

if __name__ == '__main__':
    main()
